local VERSION = tonumber(("$Revision: 40958 $"):match("%d+"))

CowTip = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0", "FuBarPlugin-2.0", "AceHook-2.1")
local CowTip, self = CowTip, CowTip
CowTip.version = "r" .. VERSION
CowTip.revision = VERSION
CowTip.date = ("$Date: 2007-06-21 00:21:08 -0400 (Thu, 21 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")

local SharedMedia = AceLibrary("SharedMedia-1.0")
local DogTag = AceLibrary("DogTag-1.0")

local _G = _G
local GameTooltip = _G.GameTooltip
local UIParent = _G.UIParent

CowTip:RegisterDB("CowTipDB")
CowTip:RegisterDefaults('profile', {
	lines = {
		lefts = {
			['*'] = "",
			"[Name]", -- [1]
			"Target:", -- [2]
			"Guild:", -- [3]
			"Guild rank:", -- [4]
			"Level:", -- [5]
			"Race:", -- [6]
			"Class:", -- [7]
			"Faction:", -- [8]
			"Status:", -- [9]
			"Health:", -- [10]
			"[~HasNoMP?Text([TypePower]:)]", -- [11]
			"[CurXP:HideZero?Text(Experience:)]", -- [12]
			"[Threat:HideZero?Text(Threat:)]", -- [13]
			"Happiness:", -- [14]
		},
		rights = {
			['*'] = "",
			"", -- [1]
			"[Target]", -- [2]
			"[Guild:Angle]", -- [3]
			"[GuildRank]", -- [4]
			"[Classification] [Level]", -- [5]
			"[SmartRace]", -- [6]
			"[SmartClass]", -- [7]
			"[Faction]", -- [8]
			"[Status]", -- [9]
			"[FractionalHP:Short] [PercentHP:Percent:Paren]", -- [10]
			"[FractionalMP:Short] [PercentMP:Percent:Paren]", -- [11]
			"[FractionalXP:Short] [PercentXP:Percent:Paren]", -- [12]
			"[FractionalThreat:Short] [PercentThreat:Percent:Paren]", -- [13]
			"[HappyText]", -- [14]
		}
	}
})

-- FuBar configuration
CowTip.hasIcon = true
CowTip.hideWithoutStandby = true
CowTip.defaultPosition = "MINIMAP"
CowTip.defaultMinimapPosition = 230
CowTip.hasNoText = true
CowTip.hasNoColor = true
CowTip.cannotDetachTooltip = true
CowTip.independentProfile = true
CowTip.blizzardTooltip = true

function CowTip:OnInitialize()
	self:RegisterChatCommand('/CowTip', '/Cow', {
		type = 'execute',
		func = function()
			self:OpenMenu()
		end
	})
end

function CowTip:OnEnable()
--	self:HookScript(GameTooltip, "OnShow", "GameTooltip_OnShow")
	self:HookScript(GameTooltip, "OnHide", "GameTooltip_OnHide")
	self:HookScript(GameTooltip, "OnTooltipSetUnit", "GameTooltip_OnTooltipSetUnit")
end

function CowTip:OnDisable()
	self:GameTooltip_OnHide(GameTooltip)
end

local lastNumRegistered = 0
function CowTip:GameTooltip_OnTooltipSetUnit(GameTooltip, ...)
--	self.hooks[this].OnTooltipSetUnit(GameTooltip, unit, ...)
--	do return end
	
	local name, unit = GameTooltip:GetUnit()
	local mouseover = unit == "mouseover"
	
	GameTooltip:ClearLines()
	local db = self.db.profile.lines
	local lefts, rights = db.lefts, db.rights
	local num = 0
	for i = 1, #lefts do
		local left, right = lefts[i], rights[i]
		local left_ev, right_ev
		if left == "" then
			left = nil
		else
			left_ev = DogTag:Evaluate(unit, left)
			if left_ev == "" then
				left_ev = nil
			end
		end
		if right == "" then
			right = nil
		else
			right_ev = DogTag:Evaluate(unit, right)
			if right_ev == "" then
				right_ev = nil
			end
		end
		if right then
			if right_ev and left and left_ev then
				num = num + 1
				GameTooltip:AddDoubleLine(left_ev or '', right_ev, 1, 1, 1, 1, 1, 1)
				local leftFS = _G["GameTooltipTextLeft" .. num]
				local rightFS = _G["GameTooltipTextRight" .. num]
				DogTag:RegisterFontString(leftFS, GameTooltip, unit, left or '')
				DogTag:RegisterFontString(rightFS, GameTooltip, unit, right)
			end
		elseif left then
			if left_ev then
				num = num + 1
				GameTooltip:AddLine(left_ev, 1, 1, 1, 1, 1, 1)
				local leftFS = _G["GameTooltipTextLeft" .. num]
				DogTag:RegisterFontString(leftFS, GameTooltip, unit, left)
			end
		end
	end
	lastNumRegistered = num
end
--[[
function CowTip:GameTooltip_OnShow(this, ...)
	self.hooks[this].OnShow(this, ...)
	
	AceLibrary("AceConsole-2.0"):Print("OnShow")
end
]]
function CowTip:GameTooltip_OnHide(this, ...)
	for i = 1, lastNumRegistered do
		DogTag:UnregisterFontString(_G["GameTooltipTextLeft" .. i])
		DogTag:UnregisterFontString(_G["GameTooltipTextRight" .. i])
	end
	lastNumRegistered = 0
	
	self.hooks[this].OnHide(this, ...)
end

CowTip.options = {
	type = 'group',
	handler = CowTip,
	args = {
		lines = {
			name = "Lines",
			desc = "Here you can describe the text on the assorted lines of the tooltip.",
			type = 'group',
			args = {
				
			}
		}
	}
}
local function getLeft(i)
	return CowTip.db.profile.lines.lefts[i]
end
local function setLeft(i, value)
	CowTip.db.profile.lines.lefts[i] = value
end
local function getRight(i)
	return CowTip.db.profile.lines.rights[i]
end
local function setRight(i, value)
	CowTip.db.profile.lines.rights[i] = value
end
for i = 1, 30 do
	CowTip.options.args.lines.args[i] = {
		name = ("Line #%d"):format(i),
		desc = ("Edit line #%d"):format(i),
		type = 'group',
		args = {
			left = {
				name = "Left",
				desc = "Left",
				type = 'text',
				usage = "DogTag-1.0 tags. See http://www.wowace.com/wiki/DogTag-1.0",
				get = getLeft,
				set = setLeft,
				passValue = i,
			},
			right = {
				name = "Right",
				desc = "Right",
				type = 'text',
				usage = "DogTag-1.0 tags. See http://www.wowace.com/wiki/DogTag-1.0",
				get = getRight,
				set = setRight,
				passValue = i,
			}
		},
		order = i,
	}
end
CowTip.OnMenuRequest = CowTip.options

function CowTip:OnTooltipUpdate()
	GameTooltip:AddLine("CowTip")
	GameTooltip:AddLine(" ")
	--[[
	GameTooltip:AddDoubleLine(LOCKED .. ":", (self.db.profile.settings.locked and YES or NO), 1, 1, 0, 1, 1, 1)
	local text
	if configMode == "solo" then
		text = L["Solo"]
	elseif configMode == "party" then
		text = L["Party"]
	elseif configMode == "raid" then
		text = L["Raid"]
	else
		text = L["Off"]
	end
	GameTooltip:AddDoubleLine(L["Configuration mode"] .. ":", text, 1, 1, 0, 1, 1, 1)
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine(L["|cffffff00Click|r to lock/unlock unit frames."], 0.2, 1, 0.2)
	GameTooltip:AddLine(L["|cffffff00Shift-Click|r to change configuration mode."], 0.2, 1, 0.2)
	]]
	GameTooltip:AddLine("|cffffff00Right-Click|r to change settings.", 0.2, 1, 0.2)
end
